Attribute VB_Name = "module1"
Option Explicit

'The SQL connection string used for ADO connections to the Unity database.  This is set in the form load routine on
'the frmMain form.
Global strConnectionString As String

'WinAPI call used to give up CPU cycles to other apps on the system for a bit.  I use this in tight loops to make sure we
'don't completely peg the CPU.
Public Declare Sub Sleep Lib "kernel32" (ByVal dwMilliseconds As Long)

'routine to clean up when we exit.  Since there wont be any open record sets or log files or the like here
'and I don't want to give the user the option of changing their mind, there's really not much to do here
'other than shut the application down.
Sub myFinalExit()
    End
End Sub


'This function calls a WinAPI routine to generate a SID string - this is used by the createSubscriber routine such that
'we can provide a SID value used to track the progress of a synchronization request after you added the subscriber to
'the SQL database.
Public Function generateDirectorySyncTicket() As String
    Dim typeLib As Variant
    
    Set typeLib = CreateObject("Scriptlet.TypeLib")
        
    generateDirectorySyncTicket = typeLib.Guid
    'remove the trailing
    generateDirectorySyncTicket = Left(generateDirectorySyncTicket, Len(generateDirectorySyncTicket) - 2)
    Set typeLib = Nothing
End Function


'This function is a handy helper when you're forcing synchronization of objects or changes into the directory.  You
'pass in the synch ticket here and it'll block while giving up CPU cycles to other processes until the ticket shows
'up as being complete.
'it'll wait for up to 40 seconds and then return a failure - false means it failed to complete or the SQLSyncSvr
'returned an error in the result column.  If it gets the ticket back in under 40 seconds and no error is reported
'the function returns true.
'This function also takes care of cleaning the ticket up for you.
Public Function WaitForSyncToFinish(strSyncTicket As String)
Dim rsSyncTicket As ADODB.Recordset
Dim iCounter As Integer
Dim oCommand As ADODB.Command
    
    On Error GoTo WaitForSyncToFinished_Error

    Set rsSyncTicket = New ADODB.Recordset
    Set oCommand = New ADODB.Command
    oCommand.ActiveConnection = strConnectionString
    oCommand.CommandType = adCmdStoredProc
    
    For iCounter = 1 To 20
        Sleep 2000 'give up the CPU to other processes for 2 seconds.
        rsSyncTicket.Open "SELECT * FROM vw_DirectorySyncTicket WHERE Ticket='" + strSyncTicket + "'", strConnectionString, adOpenKeyset, adLockReadOnly
        If rsSyncTicket.RecordCount > 0 Then
            'the record is created and then the result value is populated in it- the timing can bite you here so after we open
            'the recordset and find a match on the DirectorySyncTicket check the result value - it should never be NULL so if it
            'is, fall through and close/reopen the record set in another second.  A little awkward, but hey...
            If IsNull(rsSyncTicket("result")) = False Then
                GoTo TicketFound
            End If
        End If
        rsSyncTicket.Close
    Next iCounter
    
    'We timed out waiting for the syncronization to complete.  There's no row to remove so we can just pull the rip cord here
    WaitForSyncToFinish = False
    GoTo BailOut
    
TicketFound:
    rsSyncTicket.MoveFirst
    If rsSyncTicket("result") = 0 Then
        'you're golden, everything finished OK.
        WaitForSyncToFinish = True
    Else
        'You have not been living right and the synch failed for some reason.
        WaitForSyncToFinish = False
    End If
    
    'Since we explicitly requested a synch ticket above we need to clean up that row in the DirectorySyncTicket table
    oCommand.CommandText = "sp_DeleteDirectorySyncTicket"
    oCommand.Parameters.Item("@Ticket") = oCommand.CreateParameter("Ticket", adGUID, adParamInput, , rsSyncTicket("Ticket"))
    oCommand.Execute
    
BailOut:
    On Error Resume Next
    Set oCommand = Nothing
    rsSyncTicket.Close
    Set rsSyncTicket = Nothing
    
    Exit Function
WaitForSyncToFinished_Error:
    MsgBox "(error):" & Err.Number & " (" & Err.Description & ") in procedure WaitForSyncToFinished of Module modPMSFunctions"
    Debug.Assert False
    Resume Next
End Function

